/*
 *  Copyright 2019, Antoine Fontaine <antoine.fontaine@epfl.ch>
 *  I release this program under GNU GPLv3+.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <myhtml/myhtml.h>
#include <assert.h>
#include <stdlib.h>
#include "moodle-parser.h"
#include "data-struct.h"
#include "config.h"
#include "utils.h"

static myhtml_t *myhtml;
static gboolean caching;


char *
moodle_parse_form_error (const char *html, size_t html_size)
{
  /*myhtml_collection_t *collection = myhtml_get_nodes_by_tag_id (tree, NULL, MyHTML_TAG_SCRIPT, NULL);

  if(collection && collection->list && collection->length) {
    myhtml_tree_node_t *text_node = myhtml_node_child (collection->list[0]);

    if(text_node) {
      const char* text = myhtml_node_text (text_node, NULL);

      if(text)
        printf ("Title: %s\n", text);
    }
  }

  g_print ("No tequila problem.");

    // release resources
  myhtml_collection_destroy (collection);*/

  return NULL;
}


struct Courses
moodle_parse_course_list (const char *html, size_t html_size, const gchar *filename)
{
  myhtml_tree_t *tree;
  myhtml_tree_node_t *courses_node; // the node we will look for urls into
  myhtml_collection_t *collection;
  myhtml_collection_t *hrefs;
  struct Course *courses;

  if (!html_size || !html)
    g_error ("empty html given");

  if (filename && caching)
    write_file (filename, html, html_size);

  tree = myhtml_tree_create ();
  myhtml_tree_init (tree, myhtml);
  myhtml_parse (tree, MyENCODING_UTF_8, html, html_size);

  collection = myhtml_get_nodes_by_attribute_value (tree, NULL, NULL, FALSE,
                                                    "id", strlen("id"),
                                                    "coc-courselist", strlen("coc-courselist"),
                                                    NULL);

  if(collection && collection->list && collection->length==1)
    courses_node = collection->list[0];
  else
    g_error ("weird result getting the courses container\n");
  myhtml_collection_destroy (collection);

  /*hrefs = myhtml_get_nodes_by_attribute_value_begin (tree, NULL, courses_node, FALSE,
                                                     "href", sizeof("href"),
                                                     EPFL_COURSE_LINK, sizeof(EPFL_COURSE_LINK),
                                                     NULL);*/
  hrefs = myhtml_get_nodes_by_tag_id_in_scope (tree, NULL,
                                               courses_node,
                                               MyHTML_TAG_A,
                                               NULL);

  if (!hrefs || !hrefs->list || !hrefs->length)
    g_error ("weird result getting courses links\n");

  courses = malloc (hrefs->length*sizeof(struct Course)); // at most this size will be used.
  size_t count=0;
  size_t size;
  const char *string;
  for (size_t i=0; i<hrefs->length; i++) {
    myhtml_tree_attr_t* attr = myhtml_node_attribute_first (hrefs->list[i]);
    string = NULL;
    while (attr) {
      // <a> must have an href
      if (strcmp (myhtml_attribute_key (attr, NULL), "href") != 0) {
        attr = myhtml_attribute_next (attr);
        continue; // not the right attribute
      }
      // and href musts start with …/course/view.php?id=
      if (strncmp (myhtml_attribute_value (attr, NULL), EPFL_COURSE_LINK, strlen (EPFL_COURSE_LINK)) != 0)
        break; // not an interesting link

      string = myhtml_attribute_value (attr, &size); // yay!
      attr = myhtml_attribute_next (attr);
    }
    if (string && size) {
      // nice link, we add it!
      //g_print ("url: %s\n", string);
      courses[count].url = malloc (size+1);
      strcpy (courses[count].url, string);
      //courses[count].url[size] = '\0';

      sscanf(string, EPFL_COURSE_LINK"%u", &(courses[count].id));

      string = myhtml_node_text (myhtml_node_child (hrefs->list[i]), &size);
      //g_print (", %s\n", string);
      if (G_UNLIKELY (!string || !size)) {
        g_print ("failed to get course name\n");
        free (courses[count].url);
        continue;
      }
      courses[count].name = malloc (size+1);
      strcpy (courses[count].name, string);
      count++;
    }
  }
  courses = realloc (courses, sizeof(struct Course)*count);

  myhtml_collection_destroy (hrefs);
  myhtml_tree_destroy (tree);

  /*
  struct Course *cs = malloc(10*(sizeof(struct Course)));
  cs[0].name = malloc (50*sizeof (char));
  cs[1].name = malloc (50*sizeof (char));
  cs[2].name = malloc (50*sizeof (char));
  cs[3].name = malloc (50*sizeof (char));
  cs[4].name = malloc (50*sizeof (char));
  cs[5].name = malloc (50*sizeof (char));
  cs[6].name = malloc (50*sizeof (char));
  cs[7].name = malloc (50*sizeof (char));
  cs[8].name = malloc (50*sizeof (char));
  cs[9].name = malloc (50*sizeof (char));

  cs[0].url = malloc (50*sizeof (char));
  cs[1].url = malloc (50*sizeof (char));
  cs[2].url = malloc (50*sizeof (char));
  cs[3].url = malloc (50*sizeof (char));
  cs[4].url = malloc (50*sizeof (char));
  cs[5].url = malloc (50*sizeof (char));
  cs[6].url = malloc (50*sizeof (char));
  cs[7].url = malloc (50*sizeof (char));
  cs[8].url = malloc (50*sizeof (char));
  cs[9].url = malloc (50*sizeof (char));

  strcpy(cs[0].name, "Électricité"           );
  strcpy(cs[1].name, "Analyse"               );
  strcpy(cs[2].name, "Construction Mécanique");
  strcpy(cs[3].name, "Design of Experiment"  );
  strcpy(cs[4].name, "Électrotechnique"      );
  strcpy(cs[5].name, "Éléments de Machine"   );
  strcpy(cs[6].name, "Mobilité"              );
  strcpy(cs[7].name, "Matériaux"             );
  strcpy(cs[8].name, "Physique"              );
  strcpy(cs[9].name, "Programmation"         );

  strcpy(cs[0].url, "https://moodle.epfl.ch/course/view.php?id=14019");
  strcpy(cs[1].url, "https://moodle.epfl.ch/course/view.php?id=14837");
  strcpy(cs[2].url, "https://moodle.epfl.ch/course/view.php?id=15000");
  strcpy(cs[3].url, "https://moodle.epfl.ch/course/view.php?id=15829");
  strcpy(cs[4].url, "https://moodle.epfl.ch/course/view.php?id=15544");
  strcpy(cs[5].url, "https://moodle.epfl.ch/course/view.php?id=13923");
  strcpy(cs[6].url, "https://moodle.epfl.ch/course/view.php?id=14412");
  strcpy(cs[7].url, "https://moodle.epfl.ch/course/view.php?id=15843");
  strcpy(cs[8].url, "https://moodle.epfl.ch/course/view.php?id=13917");
  strcpy(cs[9].url, "https://moodle.epfl.ch/course/view.php?id=15698");

  cs[0].id = 14019;
  cs[1].id = 14837;
  */

  struct Courses courses_struct = {courses, count};
  return courses_struct;
}


static struct Week
moodle_extract_week (guint week_number, myhtml_tree_t *tree, myhtml_tree_node_t *div)
{
  myhtml_tree_node_t *element = div->child;
  myhtml_tree_node_t *li;
  guint file_count = 0;
  struct DownloadableFile *files = NULL;
  while (element) {
    if (element->tag_id == MyHTML_TAG_UL) {
      // files
      files = malloc (sizeof (struct DownloadableFile)*1000);
      li = element->child;
      while (li) {
        char *filename;
        char *filetextname;
        guint id;
        myhtml_collection_t *hrefs = myhtml_get_nodes_by_tag_id_in_scope (tree, NULL,
                                                                          li,
                                                                          MyHTML_TAG_A,
                                                                          NULL);
        if (!hrefs || !hrefs->length) {
          myhtml_collection_destroy (hrefs);
          li = li->next;
          continue;
        }

        myhtml_tree_node_t *node = hrefs->list[0];
        myhtml_tree_attr_t *attr = myhtml_node_attribute_first (node);
        const char *string, *href=NULL;
        size_t size;
        filename = filetextname = NULL;
        while (attr) {
          // <a> must have an href
          if (strcmp (myhtml_attribute_key (attr, NULL), "href") == 0) {
            href = myhtml_attribute_value (attr, &size);

            sscanf (href, EPFL_FILE_LINK"%u", &id);

            string = g_path_get_basename (href);
            size = strlen (string);
            filename = malloc (size+1);
            strcpy (filename, string);

            string = myhtml_node_text(node->child->next->child, &size);
            filetextname = malloc (size+1);
            strcpy(filetextname, string);
            break;
          }
          attr = myhtml_attribute_next (attr);
        }

        // and href must be a file
        if (href && strncmp (href, EPFL_FILE_LINK, strlen (EPFL_FILE_LINK)) != 0) {
          myhtml_collection_destroy (hrefs);
          li = li->next;
          continue; // not an interesting link
        }

        if (filetextname && filename) {
          assert(file_count<1000); // there should be no more than 1000 files a week, right ? right ??
          struct DownloadableFile file = {filetextname, filename, id};
          files[file_count] = file;
          file_count++;
        } else g_print ("a li doesn't have any link\n");
        myhtml_collection_destroy (hrefs);
        li = li->next;
      } while (li);
      files = realloc (files, sizeof(struct DownloadableFile)*file_count);
    }
    element = element->next;
  }

  struct Week week = {week_number, file_count, files};
  return week;
}


struct CourseContent
moodle_parse_course_content (const char *html, size_t html_size, const char *filename)
{
  myhtml_tree_t *tree;
  myhtml_collection_t *root_col;
  myhtml_tree_node_t *root_node; // the node we will look for urls into
  myhtml_collection_t *weeks_ul_col;
  myhtml_tree_node_t *weeks_ul_node;
  myhtml_tree_node_t *week_current_li_node;
  guint week_count;
  struct Week *weeks;

  if (!html_size || !html)
    g_error ("empty html given\n");

  if (filename && caching)
    write_file (filename, html, html_size);

  tree = myhtml_tree_create ();
  myhtml_tree_init (tree, myhtml);
  myhtml_parse (tree, MyENCODING_UTF_8, html, html_size);

  root_col = myhtml_get_nodes_by_attribute_value (tree, NULL, NULL, FALSE,
                                                  "role", strlen("role"),
                                                  "main", strlen("main"),
                                                  NULL);
  if(root_col && root_col->list && root_col->length==1)
    root_node = root_col->list[0];
  else
    g_error ("weird result getting the courses' root element\n");
  myhtml_collection_destroy (root_col);

  weeks_ul_col = myhtml_get_nodes_by_attribute_value (tree, NULL, root_node, FALSE,
                                                      "class", strlen("class"),
                                                      "weeks", strlen("weeks"),
                                                      NULL);

  if (!weeks_ul_col || !weeks_ul_col->list)
    g_error ("weird result getting the course's weeks'ul\n");
  if (!weeks_ul_col->length) {
    myhtml_collection_destroy (weeks_ul_col);
    g_warning ("%s seems to have no course ul\n", filename);
    struct CourseContent empty = {0, NULL};
    return empty;
  }

  weeks_ul_node = weeks_ul_col->list[0];

  weeks = malloc (sizeof (struct Week)*1000);

  week_current_li_node = weeks_ul_node->child;
  week_count=0;
  while (week_current_li_node) {
    assert(week_count<1000); // there should be no more than 1000 weeks in a semester, right ? right ??
    // each li node contain several div. We need to find the right one
    myhtml_tree_node_t *div = week_current_li_node->child;
    while (div) {
      myhtml_tree_attr_t* attr = myhtml_node_attribute_first (div);
      while (attr) {
        if (strcmp(myhtml_attribute_key (attr, NULL), "class") == 0
         && strcmp(myhtml_attribute_value (attr, NULL), "content") == 0) {
          // we reached the actual week content !
          weeks[week_count] = moodle_extract_week (week_count, tree, div);
          week_count++;
        }
        attr = attr->next;
      } // while (attr)
      div = div->next;
    }
    week_current_li_node = week_current_li_node->next;
  }
  myhtml_collection_destroy (weeks_ul_col);
  myhtml_tree_destroy (tree);
  weeks = realloc (weeks, sizeof(struct Week)*week_count);
/*
  struct DownloadableFile *fs = malloc(6*sizeof(struct DownloadableFile));
  fs[0].name = malloc (50*sizeof (char));
  fs[1].name = malloc (50*sizeof (char));
  fs[2].name = malloc (50*sizeof (char));
  fs[3].name = malloc (50*sizeof (char));
  fs[4].name = malloc (50*sizeof (char));
  fs[5].name = malloc (50*sizeof (char));

  fs[0].url = malloc (50*sizeof (char));
  fs[1].url = malloc (50*sizeof (char));
  fs[2].url = malloc (50*sizeof (char));
  fs[3].url = malloc (50*sizeof (char));
  fs[4].url = malloc (50*sizeof (char));
  fs[5].url = malloc (50*sizeof (char));

  strcpy(fs[0].name, "Analyse court complet II");
  strcpy(fs[1].name, "Série 1"                 );
  strcpy(fs[2].name, "Série 1 — corrigé"       );
  strcpy(fs[3].name, "Série 2"                 );
  strcpy(fs[4].name, "Série 2 — corrigé"       );
  strcpy(fs[5].name, "Maple file"              );

  strcpy(fs[0].url, "https://moodle.epfl.ch/course/view.php?id=14019");
  strcpy(fs[1].url, "https://moodle.epfl.ch/course/view.php?id=14837");
  strcpy(fs[2].url, "https://moodle.epfl.ch/course/view.php?id=15000");
  strcpy(fs[3].url, "https://moodle.epfl.ch/course/view.php?id=15829");
  strcpy(fs[4].url, "https://moodle.epfl.ch/course/view.php?id=15544");
  strcpy(fs[5].url, "https://moodle.epfl.ch/course/view.php?id=13923");

  struct Week ws[] =
    {{0, 5, &fs[0]},
     {1, 2, &fs[3]},
     {2, 1, &fs[5]},
     {3, 1, &fs[1]},
     {4, 1, &fs[3]},
     {5, 1, &fs[5]}};
  weeks = malloc(6*sizeof(struct Week));
  weeks[0] = ws[0];
  weeks[1] = ws[1];
  weeks[2] = ws[2];
  weeks[3] = ws[3];
  weeks[4] = ws[4];
  weeks[5] = ws[5];

  week_count = 6;
*/
  struct CourseContent content = {week_count, weeks};
  return content;
}


void
moodle_parser_course_content_free (struct CourseContent content)
{
  for (size_t i=0; i<content.week_count; i++) {
    for (size_t j=0; j<content.weeks[i].file_count; j++) {
      free (content.weeks[i].files[j].textname);
      free (content.weeks[i].files[j].name);
    }
    free (content.weeks[i].files);
  }
  free (content.weeks);
}


void
moodle_parser_init (gboolean is_caching)
{
  caching = is_caching;
  myhtml = myhtml_create ();
  myhtml_init (myhtml, MyHTML_OPTIONS_DEFAULT, 2, 0);
}

